@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief netopeer2-cli configuration
  *
- * Copyright (c) 2015 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -14,16 +16,16 @@
 
 #define _GNU_SOURCE
 #include <assert.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <pwd.h>
-#include <fcntl.h>
 #include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include <libyang/libyang.h>
 #include <nc_client.h>
@@ -32,13 +34,13 @@
 #define eaccess access
 #endif
 
+#include "commands.h"
 #include "compat.h"
 #include "configuration.h"
-#include "commands.h"
 #include "linenoise/linenoise.h"
 
 extern LYD_FORMAT output_format;
-extern int output_flag;
+extern uint32_t output_flag;
 extern char *config_editor;
 
 /* NetConf Client home (appended to ~/) */
@@ -95,14 +97,14 @@ get_default_client_cert(char **cert, char **key)
     }
 
     // trying to use *.crt and *.key format
-    if (asprintf(cert, "%s/%s", netconf_dir, CERT_CRT) == -1 || asprintf(key, "%s/%s", netconf_dir, CERT_KEY) == -1) {
+    if ((asprintf(cert, "%s/%s", netconf_dir, CERT_CRT) == -1) || (asprintf(key, "%s/%s", netconf_dir, CERT_KEY) == -1)) {
         ERROR("get_default_client_cert", "asprintf() failed (%s:%d).", __FILE__, __LINE__);
         ERROR("get_default_client_cert", "Unable to use the default client certificate due to the previous error.");
         free(netconf_dir);
         return;
     }
 
-    if (eaccess(*cert, R_OK) == -1 || eaccess(*key, R_OK) == -1) {
+    if ((eaccess(*cert, R_OK) == -1) || (eaccess(*key, R_OK) == -1)) {
         if (errno != ENOENT) {
             ERROR("get_default_client_cert", "Unable to access \"%s\" and \"%s\": %s", *cert, *key, strerror(errno));
             free(*key);
@@ -250,12 +252,12 @@ void
 load_config(void)
 {
     char *netconf_dir, *history_file, *config_file = NULL;
-    struct lyxml_elem *config_xml = NULL, *child;
+    struct lyd_node *config = NULL, *child;
     struct ly_ctx *ctx;
 
 #ifdef NC_ENABLED_SSH
     const char *key_pub, *key_priv;
-    struct lyxml_elem *auth_child, *pref_child, *key_child, *pair_child;
+    struct lyd_node *auth_child, *pref_child, *key_child, *pair_child;
 #endif
 
     if ((netconf_dir = get_netconf_dir()) == NULL) {
@@ -274,8 +276,7 @@ load_config(void)
         }
     }
 
-    ctx = ly_ctx_new(NULL, 0);
-    if (!ctx) {
+    if (ly_ctx_new(NULL, 0, &ctx)) {
         ERROR(__func__, "Failed to create context.");
         ERROR(__func__, "Unable to load configuration due to the previous error.");
         ctx = NULL;
@@ -287,67 +288,67 @@ load_config(void)
         } else if (eaccess(config_file, F_OK) && (errno == ENOENT)) {
             ERROR(__func__, "No saved configuration.");
         } else {
-            if ((config_xml = lyxml_parse_path(ctx, config_file, 0)) == NULL) {
+            if (lyd_parse_data_path(ctx, config_file, LYD_XML, LYD_PARSE_ONLY | LYD_PARSE_OPAQ, 0, &config)) {
                 ERROR(__func__, "Failed to load configuration of NETCONF client (lyxml_read_path failed).");
-                config_xml = NULL;
+                config = NULL;
             } else {
                 /* doc -> <netconf-client/>*/
-                if (!strcmp(config_xml->name, "netconf-client")) {
-                    LY_TREE_FOR(config_xml->child, child) {
-                        if (!strcmp(child->name, "editor")) {
+                if (!strcmp(LYD_NAME(config), "netconf-client")) {
+                    LY_LIST_FOR(lyd_child(config), child) {
+                        if (!strcmp(LYD_NAME(child), "editor")) {
                             /* doc -> <netconf-client> -> <editor> */
                             if (config_editor) {
                                 free(config_editor);
                             }
-                            config_editor = strdup(child->content);
-                        } else if (!strcmp(child->name, "searchpath")) {
+                            config_editor = strdup(lyd_get_value(child));
+                        } else if (!strcmp(LYD_NAME(child), "searchpath")) {
                             /* doc -> <netconf-client> -> <searchpath> */
                             errno = 0;
-                            if (!mkdir(child->content, 00700) || (errno == EEXIST)) {
+                            if (!mkdir(lyd_get_value(child), 00700) || (errno == EEXIST)) {
                                 if (errno == 0) {
-                                    ERROR(__func__, "Search path \"%s\" did not exist, created.", child->content);
+                                    ERROR(__func__, "Search path \"%s\" did not exist, created.", lyd_get_value(child));
                                 }
-                                nc_client_set_schema_searchpath(child->content);
+                                nc_client_set_schema_searchpath(lyd_get_value(child));
                             } else {
-                                ERROR(__func__, "Search path \"%s\" cannot be created: %s", child->content, strerror(errno));
+                                ERROR(__func__, "Search path \"%s\" cannot be created: %s", lyd_get_value(child), strerror(errno));
                             }
-                        } else if (!strcmp(child->name, "output-format")) {
+                        } else if (!strcmp(LYD_NAME(child), "output-format")) {
                             /* doc -> <netconf-client> -> <output-format> */
-                            if (!strcmp(child->content, "json")) {
-                                output_format = LYD_JSON;
-                                output_flag = LYP_FORMAT;
-                            } else if (!strcmp(child->content, "json_noformat")) {
+                            if (!strcmp(lyd_get_value(child), "json")) {
                                 output_format = LYD_JSON;
                                 output_flag = 0;
-                            } else if (!strcmp(child->content, "xml_noformat")) {
+                            } else if (!strcmp(lyd_get_value(child), "json_noformat")) {
+                                output_format = LYD_JSON;
+                                output_flag = LYD_PRINT_SHRINK;
+                            } else if (!strcmp(lyd_get_value(child), "xml_noformat")) {
                                 output_format = LYD_XML;
-                                output_flag = 0;
+                                output_flag = LYD_PRINT_SHRINK;
                             } /* else default (formatted XML) */
                         }
 #ifdef NC_ENABLED_SSH
-                        else if (!strcmp(child->name, "authentication")) {
+                        else if (!strcmp(LYD_NAME(child), "authentication")) {
                             /* doc -> <netconf-client> -> <authentication> */
-                            LY_TREE_FOR(child->child, auth_child) {
-                                if (!strcmp(auth_child->name, "pref")) {
-                                    LY_TREE_FOR(auth_child->child, pref_child) {
-                                        if (!strcmp(pref_child->name, "publickey")) {
-                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_PUBLICKEY, atoi(pref_child->content));
-                                        } else if (!strcmp(pref_child->name, "interactive")) {
-                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_INTERACTIVE, atoi(pref_child->content));
-                                        } else if (!strcmp(pref_child->name, "password")) {
-                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_PASSWORD, atoi(pref_child->content));
+                            LY_LIST_FOR(lyd_child(child), auth_child) {
+                                if (!strcmp(LYD_NAME(auth_child), "pref")) {
+                                    LY_LIST_FOR(lyd_child(auth_child), pref_child) {
+                                        if (!strcmp(LYD_NAME(pref_child), "publickey")) {
+                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_PUBLICKEY, atoi(lyd_get_value(pref_child)));
+                                        } else if (!strcmp(LYD_NAME(pref_child), "interactive")) {
+                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_INTERACTIVE, atoi(lyd_get_value(pref_child)));
+                                        } else if (!strcmp(LYD_NAME(pref_child), "password")) {
+                                            nc_client_ssh_set_auth_pref(NC_SSH_AUTH_PASSWORD, atoi(lyd_get_value(pref_child)));
                                         }
                                     }
-                                } else if (!strcmp(auth_child->name, "keys")) {
-                                    LY_TREE_FOR(auth_child->child, key_child) {
-                                        if (!strcmp(key_child->name, "pair")) {
+                                } else if (!strcmp(LYD_NAME(auth_child), "keys")) {
+                                    LY_LIST_FOR(lyd_child(auth_child), key_child) {
+                                        if (!strcmp(LYD_NAME(key_child), "pair")) {
                                             key_pub = NULL;
                                             key_priv = NULL;
-                                            LY_TREE_FOR(key_child->child, pair_child) {
-                                                if (!strcmp(pair_child->name, "public")) {
-                                                    key_pub = pair_child->content;
-                                                } else if (!strcmp(pair_child->name, "private")) {
-                                                    key_priv = pair_child->content;
+                                            LY_LIST_FOR(lyd_child(key_child), pair_child) {
+                                                if (!strcmp(LYD_NAME(pair_child), "public")) {
+                                                    key_pub = lyd_get_value(pair_child);
+                                                } else if (!strcmp(LYD_NAME(pair_child), "private")) {
+                                                    key_priv = lyd_get_value(pair_child);
                                                 }
                                             }
                                             if (key_pub && key_priv) {
@@ -366,8 +367,8 @@ load_config(void)
         }
     }
 
-    lyxml_free(ctx, config_xml);
-    ly_ctx_destroy(ctx, NULL);
+    lyd_free_tree(config);
+    ly_ctx_destroy(ctx);
     free(config_file);
     free(history_file);
     free(netconf_dir);
@@ -403,7 +404,7 @@ store_config(void)
         ERROR(__func__, "Unable to store configuration due to the previous error.");
     } else {
         indent = 0;
-        fprintf(config_f, "%*.s<netconf-client>\n", indent, "");
+        fprintf(config_f, "%*.s<netconf-client xmlns=\"urn:cesnet:netconf-client\">\n", indent, "");
         ++indent;
 
         /* editor */
@@ -418,15 +419,15 @@ store_config(void)
         fprintf(config_f, "%*.s<output-format>", indent, "");
         if (output_format == LYD_JSON) {
             if (output_flag) {
-                fprintf(config_f, "json");
-            } else {
                 fprintf(config_f, "json_noformat");
+            } else {
+                fprintf(config_f, "json");
             }
         } else {
             if (output_flag) {
-                fprintf(config_f, "xml");
-            } else {
                 fprintf(config_f, "xml_noformat");
+            } else {
+                fprintf(config_f, "xml");
             }
         }
         fprintf(config_f, "</output-format>\n");
