@@ -24,7 +24,7 @@ GROUP=${NP2_MODULE_GROUP}
 # array of modules to install
 MODULES=(
 "ietf-netconf-acm@2018-02-14.yang"
-"ietf-netconf@2013-09-29.yang -e writable-running -e candidate -e rollback-on-error -e validate -e startup -e url -e xpath"
+"ietf-netconf@2013-09-29.yang -e writable-running -e candidate -e rollback-on-error -e validate -e startup -e url -e xpath -e confirmed-commit"
 "ietf-netconf-monitoring@2010-10-04.yang"
 "ietf-netconf-nmda@2019-01-07.yang -e origin -e with-defaults"
 "nc-notifications@2008-07-14.yang"
@@ -37,6 +37,11 @@ MODULES=(
 "ietf-ssh-server@2019-07-02.yang -e local-client-auth-supported"
 "ietf-tls-server@2019-07-02.yang -e local-client-auth-supported"
 "ietf-netconf-server@2019-07-02.yang -e ssh-listen -e tls-listen -e ssh-call-home -e tls-call-home"
+"ietf-interfaces@2018-02-20.yang"
+"ietf-ip@2018-02-22.yang"
+"ietf-network-instance@2019-01-21.yang"
+"ietf-subscribed-notifications@2019-09-09.yang -e encode-xml -e replay -e subtree -e xpath"
+"ietf-yang-push@2019-09-09.yang -e on-change"
 )
 
 # functions
@@ -70,6 +75,21 @@ UPDATE_MODULE() {
     fi
 }
 
+CHANGE_PERMS() {
+    CMD="'$SYSREPOCTL' -c $1 -p '$PERMS' -v2"
+    if [ ! -z ${OWNER} ]; then
+        CMD="$CMD -o '$OWNER'"
+    fi
+    if [ ! -z ${GROUP} ]; then
+        CMD="$CMD -g '$GROUP'"
+    fi
+    eval $CMD
+    local rc=$?
+    if [ $rc -ne 0 ]; then
+        exit $rc
+    fi
+}
+
 ENABLE_FEATURE() {
     "$SYSREPOCTL" -a -c $1 -e $2 -v2
     local rc=$?
@@ -99,6 +119,14 @@ for i in "${MODULES[@]}"; do
         UPDATE_MODULE "$file"
     fi
 
+    sctl_owner=`echo "$SCTL_MODULE" | sed 's/\([^|]*|\)\{3\} \([^:]*\).*/\2/'`
+    sctl_group=`echo "$SCTL_MODULE" | sed 's/\([^|]*|\)\{3\}[^:]*:\([^ ]*\).*/\2/'`
+    sctl_perms=`echo "$SCTL_MODULE" | sed 's/\([^|]*|\)\{4\} \([^ ]*\).*/\2/'`
+    if [ "$sctl_perms" != "$PERMS" ] || [ ! -z ${OWNER} -a "$sctl_owner" != "$OWNER" ] || [ ! -z ${GROUP} -a "$sctl_group" != "$GROUP" ]; then
+        # change permissions/owner
+        CHANGE_PERMS "$name"
+    fi
+
     # parse sysrepoctl features and add extra space at the end for easier matching
     sctl_features="`echo "$SCTL_MODULE" | sed 's/\([^|]*|\)\{6\}\(.*\)/\2/'` "
     # parse features we want to enable
