@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief netopeer2-server log functions
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -11,22 +13,25 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
-#define _DEFAULT_SOURCE
+
+#define _GNU_SOURCE
+
+#include "log.h"
 
 #include <errno.h>
 #include <stdarg.h>
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <syslog.h>
 #include <unistd.h>
 
-#include <nc_server.h>
 #include <libyang/libyang.h>
+#include <nc_server.h>
 #include <sysrepo.h>
 
 #include "common.h"
-#include "log.h"
+#include "compat.h"
 
 volatile uint8_t np2_verbose_level;
 uint8_t np2_libssh_verbose_level;
@@ -76,66 +81,14 @@ np2log(int priority, const char *src, const char *fmt, ...)
     }
 }
 
-/**
- * @brief Encode message characters (% -> %%) to avoid printf arg problems.
- */
-static const char *
-np2log_encode(const char *msg, char **buf)
-{
-    const char *ptr1, *ptr2;
-    size_t buf_len, buf_size = 1;
-    void *mem;
-
-    *buf = NULL;
-    if ((ptr2 = strchr(msg, '%'))) {
-        /* something to encode */
-        ptr1 = msg;
-        do {
-            /* enlarge buffer */
-            buf_len = buf_size - 1;
-            buf_size += (ptr2 - ptr1) + 2;
-            mem = realloc(*buf, buf_size * sizeof **buf);
-            if (!mem) {
-                EMEM;
-                return "";
-            }
-            *buf = mem;
-
-            /* copy preceding message */
-            strncpy(*buf + buf_len, ptr1, ptr2 - ptr1);
-            buf_len += ptr2 - ptr1;
-
-            /* copy % */
-            strcpy(*buf + buf_len, "%%");
-
-            /* next iter */
-            ptr1 = ptr2 + 1;
-        } while ((ptr2 = strchr(ptr1, '%')));
-
-        /* copy remaining message */
-        buf_len = buf_size - 1;
-        buf_size += strlen(ptr1);
-        mem = realloc(*buf, buf_size * sizeof **buf);
-        if (!mem) {
-            EMEM;
-            return "";
-        }
-        *buf = mem;
-        strcpy(*buf + buf_len, ptr1);
-    }
-
-    return (*buf ? *buf : msg);
-}
-
 /**
  * @brief printer callback for libnetconf2
  */
 void
-np2log_cb_nc2(NC_VERB_LEVEL level, const char *msg)
+np2log_cb_nc2(const struct nc_session *session, NC_VERB_LEVEL level, const char *msg)
 {
     int priority = LOG_ERR;
-    const char *log_msg;
-    char *buf;
+    char *buf = NULL;
 
     if (level > np2_verbose_level) {
         return;
@@ -143,7 +96,7 @@ np2log_cb_nc2(NC_VERB_LEVEL level, const char *msg)
 
     switch (level) {
     case NC_VERB_ERROR:
-        priority = LOG_ERR;;
+        priority = LOG_ERR;
         break;
     case NC_VERB_WARNING:
         priority = LOG_WARNING;
@@ -157,8 +110,12 @@ np2log_cb_nc2(NC_VERB_LEVEL level, const char *msg)
         break;
     }
 
-    log_msg = np2log_encode(msg, &buf);
-    np2log(priority, "LN", log_msg);
+    if (session && nc_session_get_id(session)) {
+        if (asprintf(&buf, "Session %u: %s", nc_session_get_id(session), msg) > -1) {
+            msg = buf;
+        }
+    }
+    np2log(priority, "LN", "%s", msg);
     free(buf);
 }
 
@@ -169,8 +126,6 @@ void
 np2log_cb_ly(LY_LOG_LEVEL level, const char *msg, const char *path)
 {
     int priority;
-    const char *log_msg;
-    char *buf;
 
     if (level > np2_verbose_level) {
         return;
@@ -197,9 +152,7 @@ np2log_cb_ly(LY_LOG_LEVEL level, const char *msg, const char *path)
     if (path) {
         np2log(priority, "LY", "%s (%s)", msg, path);
     } else {
-        log_msg = np2log_encode(msg, &buf);
-        np2log(priority, "LY", log_msg);
-        free(buf);
+        np2log(priority, "LY", "%s", msg);
     }
 }
 
@@ -207,8 +160,6 @@ void
 np2log_cb_sr(sr_log_level_t level, const char *msg)
 {
     int priority = LOG_ERR;
-    const char *log_msg;
-    char *buf;
 
     if (level > np2_sr_verbose_level) {
         return;
@@ -231,9 +182,7 @@ np2log_cb_sr(sr_log_level_t level, const char *msg)
         return;
     }
 
-    log_msg = np2log_encode(msg, &buf);
-    np2log(priority, "SR", log_msg);
-    free(buf);
+    np2log(priority, "SR", "%s", msg);
 }
 
 /**
@@ -271,7 +220,6 @@ np2log_printf(NC_VERB_LEVEL level, const char *format, ...)
         msg_len = req_len + 1;
         mem = realloc(msg, msg_len);
         if (!mem) {
-            free(msg);
             goto cleanup;
         }
         msg = mem;
@@ -285,7 +233,7 @@ np2log_printf(NC_VERB_LEVEL level, const char *format, ...)
 
     switch (level) {
     case NC_VERB_ERROR:
-        priority = LOG_ERR;;
+        priority = LOG_ERR;
         break;
     case NC_VERB_WARNING:
         priority = LOG_WARNING;
