@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-netconf-monitoring statistics and counters
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -11,20 +13,21 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
-#include <time.h>
+
+#include "netconf_monitoring.h"
+
+#include <pthread.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <pthread.h>
+#include <time.h>
 
 #include <libyang/libyang.h>
 #include <nc_server.h>
 
 #include "common.h"
+#include "compat.h"
 #include "log.h"
-#include "netconf_monitoring.h"
-
-#define NCM_TIMEZONE "CET"
 
 struct ncm stats;
 
@@ -68,7 +71,7 @@ ncm_is_monitored(struct nc_session *session)
 #ifdef NC_ENABLED_TLS
     case NC_TI_OPENSSL:
 #endif
-#if defined(NC_ENABLED_SSH) || defined(NC_ENABLED_TLS)
+#if defined (NC_ENABLED_SSH) || defined (NC_ENABLED_TLS)
         return 1;
 #endif
     default:
@@ -157,19 +160,20 @@ ncm_session_add(struct nc_session *session)
     new = realloc(stats.sessions, stats.session_count * sizeof *stats.sessions);
     if (!new) {
         EMEM;
-        return;
+        goto cleanup;
     }
     stats.sessions = new;
     new = realloc(stats.session_stats, stats.session_count * sizeof *stats.session_stats);
     if (!new) {
         EMEM;
-        return;
+        goto cleanup;
     }
     stats.session_stats = new;
 
     stats.sessions[stats.session_count - 1] = session;
     memset(&stats.session_stats[stats.session_count - 1], 0, sizeof *stats.session_stats);
 
+cleanup:
     pthread_mutex_unlock(&stats.lock);
 }
 
@@ -216,123 +220,125 @@ ncm_bad_hello(struct nc_session *session)
     pthread_mutex_unlock(&stats.lock);
 }
 
-struct lyd_node *
-ncm_get_data(sr_conn_ctx_t *conn)
+uint32_t
+ncm_session_get_notification(struct nc_session *session)
 {
-    struct lyd_node *root = NULL, *cont, *list, *cont2, *cont3;
+    uint32_t count;
+
+    if (!ncm_is_monitored(session)) {
+        return 0;
+    }
+
+    pthread_mutex_lock(&stats.lock);
+
+    count = stats.session_stats[find_session_idx(session)].out_notifications;
+
+    pthread_mutex_unlock(&stats.lock);
+
+    return count;
+}
+
+static void
+ncm_data_add_ds_lock(sr_conn_ctx_t *conn, const char *ds_str, sr_datastore_t ds, struct lyd_node *parent)
+{
+    struct lyd_node *list, *cont, *cont2;
+    char *buf, ncid_str[11];
+    int rc, is_locked;
+    uint32_t sid;
+    struct timespec ts;
+    struct nc_session *ncs;
+
+    lyd_new_list(parent, NULL, "datastore", 0, &list, ds_str);
+    rc = sr_get_lock(conn, ds, NULL, &is_locked, &sid, &ts);
+    if (rc != SR_ERR_OK) {
+        WRN("Failed to learn about %s lock (%s).", ds_str, sr_strerror(rc));
+    } else if (is_locked) {
+        lyd_new_inner(list, NULL, "locks", 0, &cont);
+        lyd_new_inner(cont, NULL, "global-lock", 0, &cont2);
+
+        np_get_nc_sess_by_id(sid, 0, &ncs);
+        sprintf(ncid_str, "%" PRIu32, ncs ? nc_session_get_id(ncs) : 0);
+        lyd_new_term(cont2, NULL, "locked-by-session", ncid_str, 0, NULL);
+
+        ly_time_ts2str(&ts, &buf);
+        lyd_new_term(cont2, NULL, "locked-time", buf, 0, NULL);
+        free(buf);
+    }
+}
+
+int
+np2srv_ncm_oper_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *UNUSED(path), const char *UNUSED(request_xpath), uint32_t UNUSED(request_id),
+        struct lyd_node **parent, void *UNUSED(private_data))
+{
+    struct lyd_node *root = NULL, *cont, *list;
     const struct lys_module *mod;
+    sr_conn_ctx_t *conn;
     struct ly_ctx *ly_ctx;
     const char **cpblts;
-    char buf[26];
-    uint32_t i, nc_id;
-    int rc, is_locked;
-    time_t ts;
+    char *time_str, buf[11];
+    uint32_t i;
 
+    conn = sr_session_get_connection(session);
     ly_ctx = (struct ly_ctx *)sr_get_context(conn);
 
-    root = lyd_new_path(NULL, ly_ctx, "/ietf-netconf-monitoring:netconf-state", NULL, 0, 0);
-    if (!root) {
+    if (lyd_new_path(NULL, ly_ctx, "/ietf-netconf-monitoring:netconf-state", NULL, 0, &root)) {
         goto error;
     }
 
     /* capabilities */
-    cont = lyd_new(root, NULL, "capabilities");
+    lyd_new_inner(root, NULL, "capabilities", 0, &cont);
 
-    cpblts = nc_server_get_cpblts_version(ly_ctx, LYS_VERSION_1);
+    cpblts = nc_server_get_cpblts_version(ly_ctx, LYS_VERSION_1_0);
     if (!cpblts) {
         goto error;
     }
 
     for (i = 0; cpblts[i]; ++i) {
-        lyd_new_leaf(cont, NULL, "capability", cpblts[i]);
+        lyd_new_term(cont, NULL, "capability", cpblts[i], 0, NULL);
         lydict_remove(ly_ctx, cpblts[i]);
     }
     free(cpblts);
 
-    cont = lyd_new(root, NULL, "datastores");
-
-    list = lyd_new(cont, NULL, "datastore");
-    lyd_new_leaf(list, NULL, "name", "running");
-    rc = sr_get_lock(conn, SR_DS_RUNNING, NULL, &is_locked, NULL, &nc_id, &ts);
-    if (rc != SR_ERR_OK) {
-        WRN("Failed to learn about running lock (%s).", sr_strerror(rc));
-    } else if (is_locked) {
-        cont2 = lyd_new(list, NULL, "locks");
-        cont3 = lyd_new(cont2, NULL, "global-lock");
-        sprintf(buf, "%u", nc_id);
-        lyd_new_leaf(cont3, NULL, "locked-by-session", buf);
-        nc_time2datetime(ts, NCM_TIMEZONE, buf);
-        lyd_new_leaf(cont3, NULL, "locked-time", buf);
-    }
-
-    list = lyd_new(cont, NULL, "datastore");
-    lyd_new_leaf(list, NULL, "name", "startup");
-    rc = sr_get_lock(conn, SR_DS_STARTUP, NULL, &is_locked, NULL, &nc_id, &ts);
-    if (rc != SR_ERR_OK) {
-        WRN("Failed to learn about startup lock (%s).", sr_strerror(rc));
-    } else if (is_locked) {
-        cont2 = lyd_new(list, NULL, "locks");
-        cont3 = lyd_new(cont2, NULL, "global-lock");
-        sprintf(buf, "%u", nc_id);
-        lyd_new_leaf(cont3, NULL, "locked-by-session", buf);
-        nc_time2datetime(ts, NCM_TIMEZONE, buf);
-        lyd_new_leaf(cont3, NULL, "locked-time", buf);
-    }
-
-    list = lyd_new(cont, NULL, "datastore");
-    lyd_new_leaf(list, NULL, "name", "candidate");
-    rc = sr_get_lock(conn, SR_DS_CANDIDATE, NULL, &is_locked, NULL, &nc_id, &ts);
-    if (rc != SR_ERR_OK) {
-        WRN("Failed to learn about candidate lock (%s).", sr_strerror(rc));
-    } else if (is_locked) {
-        cont2 = lyd_new(list, NULL, "locks");
-        cont3 = lyd_new(cont2, NULL, "global-lock");
-        sprintf(buf, "%u", nc_id);
-        lyd_new_leaf(cont3, NULL, "locked-by-session", buf);
-        nc_time2datetime(ts, NCM_TIMEZONE, buf);
-        lyd_new_leaf(cont3, NULL, "locked-time", buf);
-    }
+    /* datastore locks */
+    lyd_new_inner(root, NULL, "datastores", 0, &cont);
+    ncm_data_add_ds_lock(conn, "running", SR_DS_RUNNING, cont);
+    ncm_data_add_ds_lock(conn, "startup", SR_DS_STARTUP, cont);
+    ncm_data_add_ds_lock(conn, "candidate", SR_DS_CANDIDATE, cont);
 
     /* schemas */
-    cont = lyd_new(root, NULL, "schemas");
+    lyd_new_inner(root, NULL, "schemas", 0, &cont);
 
     i = 0;
     while ((mod = ly_ctx_get_module_iter(ly_ctx, &i))) {
-        list = lyd_new(cont, NULL, "schema");
-        lyd_new_leaf(list, NULL, "identifier", mod->name);
-        lyd_new_leaf(list, NULL, "version", (mod->rev ? mod->rev[0].date : NULL));
-        lyd_new_leaf(list, NULL, "format", "yang");
-        lyd_new_leaf(list, NULL, "namespace", lys_main_module(mod)->ns);
-        lyd_new_leaf(list, NULL, "location", "NETCONF");
+        lyd_new_list(cont, NULL, "schema", 0, &list, mod->name, mod->revision ? mod->revision : "", "yang");
+        lyd_new_term(list, NULL, "namespace", mod->ns, 0, NULL);
+        lyd_new_term(list, NULL, "location", "NETCONF", 0, NULL);
 
-        list = lyd_new(cont, NULL, "schema");
-        lyd_new_leaf(list, NULL, "identifier", mod->name);
-        lyd_new_leaf(list, NULL, "version", (mod->rev ? mod->rev[0].date : NULL));
-        lyd_new_leaf(list, NULL, "format", "yin");
-        lyd_new_leaf(list, NULL, "namespace", lys_main_module(mod)->ns);
-        lyd_new_leaf(list, NULL, "location", "NETCONF");
+        lyd_new_list(cont, NULL, "schema", 0, &list, mod->name, mod->revision ? mod->revision : "", "yin");
+        lyd_new_term(list, NULL, "namespace", mod->ns, 0, NULL);
+        lyd_new_term(list, NULL, "location", "NETCONF", 0, NULL);
     }
 
     /* sessions */
     pthread_mutex_lock(&stats.lock);
 
     if (stats.session_count) {
-        cont = lyd_new(root, NULL, "sessions");
+        lyd_new_inner(root, NULL, "sessions", 0, &cont);
 
         for (i = 0; i < stats.session_count; ++i) {
-            list = lyd_new(cont, NULL, "session");
-
             sprintf(buf, "%u", nc_session_get_id(stats.sessions[i]));
-            lyd_new_leaf(list, NULL, "session-id", buf);
+            lyd_new_list(cont, NULL, "session", 0, &list, buf);
+
             switch (nc_session_get_ti(stats.sessions[i])) {
 #ifdef NC_ENABLED_SSH
             case NC_TI_LIBSSH:
-                lyd_new_leaf(list, NULL, "transport", "netconf-ssh");
+                lyd_new_term(list, NULL, "transport", "netconf-ssh", 0, NULL);
                 break;
 #endif
 #ifdef NC_ENABLED_TLS
             case NC_TI_OPENSSL:
-                lyd_new_leaf(list, NULL, "transport", "netconf-tls");
+                lyd_new_term(list, NULL, "transport", "netconf-tls", 0, NULL);
                 break;
 #endif
             default: /* NC_TI_FD, NC_TI_NONE */
@@ -340,51 +346,54 @@ ncm_get_data(sr_conn_ctx_t *conn)
                 pthread_mutex_unlock(&stats.lock);
                 goto error;
             }
-            lyd_new_leaf(list, NULL, "username", nc_session_get_username(stats.sessions[i]));
-            lyd_new_leaf(list, NULL, "source-host", nc_session_get_host(stats.sessions[i]));
-            nc_time2datetime(nc_session_get_start_time(stats.sessions[i]), NCM_TIMEZONE, buf);
-            lyd_new_leaf(list, NULL, "login-time", buf);
+            lyd_new_term(list, NULL, "username", nc_session_get_username(stats.sessions[i]), 0, NULL);
+            lyd_new_term(list, NULL, "source-host", nc_session_get_host(stats.sessions[i]), 0, NULL);
+            ly_time_time2str(nc_session_get_start_time(stats.sessions[i]), NULL, &time_str);
+            lyd_new_term(list, NULL, "login-time", time_str, 0, NULL);
+            free(time_str);
 
             sprintf(buf, "%u", stats.session_stats[i].in_rpcs);
-            lyd_new_leaf(list, NULL, "in-rpcs", buf);
+            lyd_new_term(list, NULL, "in-rpcs", buf, 0, NULL);
             sprintf(buf, "%u", stats.session_stats[i].in_bad_rpcs);
-            lyd_new_leaf(list, NULL, "in-bad-rpcs", buf);
+            lyd_new_term(list, NULL, "in-bad-rpcs", buf, 0, NULL);
             sprintf(buf, "%u", stats.session_stats[i].out_rpc_errors);
-            lyd_new_leaf(list, NULL, "out-rpc-errors", buf);
+            lyd_new_term(list, NULL, "out-rpc-errors", buf, 0, NULL);
             sprintf(buf, "%u", stats.session_stats[i].out_notifications);
-            lyd_new_leaf(list, NULL, "out-notifications", buf);
+            lyd_new_term(list, NULL, "out-notifications", buf, 0, NULL);
         }
     }
 
     /* statistics */
-    cont = lyd_new(root, NULL, "statistics");
+    lyd_new_inner(root, NULL, "statistics", 0, &cont);
 
-    nc_time2datetime(stats.netconf_start_time, NCM_TIMEZONE, buf);
-    lyd_new_leaf(cont, NULL, "netconf-start-time", buf);
+    ly_time_time2str(stats.netconf_start_time, NULL, &time_str);
+    lyd_new_term(cont, NULL, "netconf-start-time", time_str, 0, NULL);
+    free(time_str);
     sprintf(buf, "%u", stats.in_bad_hellos);
-    lyd_new_leaf(cont, NULL, "in-bad-hellos", buf);
+    lyd_new_term(cont, NULL, "in-bad-hellos", buf, 0, NULL);
     sprintf(buf, "%u", stats.in_sessions);
-    lyd_new_leaf(cont, NULL, "in-sessions", buf);
+    lyd_new_term(cont, NULL, "in-sessions", buf, 0, NULL);
     sprintf(buf, "%u", stats.dropped_sessions);
-    lyd_new_leaf(cont, NULL, "dropped-sessions", buf);
+    lyd_new_term(cont, NULL, "dropped-sessions", buf, 0, NULL);
     sprintf(buf, "%u", stats.global_stats.in_rpcs);
-    lyd_new_leaf(cont, NULL, "in-rpcs", buf);
+    lyd_new_term(cont, NULL, "in-rpcs", buf, 0, NULL);
     sprintf(buf, "%u", stats.global_stats.in_bad_rpcs);
-    lyd_new_leaf(cont, NULL, "in-bad-rpcs", buf);
+    lyd_new_term(cont, NULL, "in-bad-rpcs", buf, 0, NULL);
     sprintf(buf, "%u", stats.global_stats.out_rpc_errors);
-    lyd_new_leaf(cont, NULL, "out-rpc-errors", buf);
+    lyd_new_term(cont, NULL, "out-rpc-errors", buf, 0, NULL);
     sprintf(buf, "%u", stats.global_stats.out_notifications);
-    lyd_new_leaf(cont, NULL, "out-notifications", buf);
+    lyd_new_term(cont, NULL, "out-notifications", buf, 0, NULL);
 
     pthread_mutex_unlock(&stats.lock);
 
-    if (lyd_validate(&root, LYD_OPT_NOSIBLINGS, NULL)) {
+    if (lyd_validate_all(&root, NULL, LYD_VALIDATE_PRESENT, NULL)) {
         goto error;
     }
 
-    return root;
+    *parent = root;
+    return SR_ERR_OK;
 
 error:
-    lyd_free(root);
-    return NULL;
+    lyd_free_tree(root);
+    return SR_ERR_INTERNAL;
 }
