@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-netconf-server callbacks
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -11,32 +13,36 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE /* asprintf() */
 
+#include "netconf_server.h"
+
+#include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <string.h>
-#include <assert.h>
 
-#include <nc_server.h>
 #include <libyang/libyang.h>
+#include <nc_server.h>
 #include <sysrepo.h>
 
 #include "common.h"
+#include "compat.h"
 #include "log.h"
 
 int
 np2srv_sr_get_privkey(const struct lyd_node *asym_key, char **privkey_data, NC_SSH_KEY_TYPE *privkey_type)
 {
-    struct lyd_node_leaf_list *alg = NULL, *privkey = NULL;
-    struct lyd_node *node;
+    struct lyd_node_term *alg = NULL;
+    struct lyd_node *node, *privkey = NULL;
 
     /* find the nodes */
-    LY_TREE_FOR(asym_key->child, node) {
+    LY_LIST_FOR(lyd_child(asym_key), node) {
         if (!strcmp(node->schema->name, "algorithm")) {
-            alg = (struct lyd_node_leaf_list *)node;
+            alg = (struct lyd_node_term *)node;
         } else if (!strcmp(node->schema->name, "private-key")) {
-            privkey = (struct lyd_node_leaf_list *)node;
+            privkey = node;
         }
     }
     if (!alg || !privkey) {
@@ -50,12 +56,12 @@ np2srv_sr_get_privkey(const struct lyd_node *asym_key, char **privkey_data, NC_S
     } else if (!strncmp(alg->value.ident->name, "secp", 4)) {
         *privkey_type = NC_SSH_KEY_ECDSA;
     } else {
-        ERR("Unknown private key algorithm \"%s\".", alg->value_str);
+        ERR("Unknown private key algorithm \"%s\".", lyd_get_value(&alg->node));
         return -1;
     }
 
     /* set data */
-    *privkey_data = strdup(privkey->value_str);
+    *privkey_data = strdup(lyd_get_value(privkey));
     if (!*privkey_data) {
         EMEM;
         return -1;
@@ -66,14 +72,12 @@ np2srv_sr_get_privkey(const struct lyd_node *asym_key, char **privkey_data, NC_S
 
 /* /ietf-netconf-server:netconf-server/listen/idle-timeout */
 int
-np2srv_idle_timeout_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_idle_timeout_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list;
-    bool prev_dflt;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -82,10 +86,10 @@ np2srv_idle_timeout_cb(sr_session_ctx_t *session, const char *UNUSED(module_name
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* ignore other operations */
         if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-            nc_server_set_idle_timeout(((struct lyd_node_leaf_list *)node)->value.uint16);
+            nc_server_set_idle_timeout(((struct lyd_node_term *)node)->value.uint16);
         }
     }
     sr_free_change_iter(iter);
@@ -103,8 +107,6 @@ np2srv_tcp_keepalives(const char *client_name, const char *endpt_name, sr_sessio
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list;
-    bool prev_dflt;
     int rc, idle_time = -1, max_probes = -1, probe_interval = -1;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -113,24 +115,24 @@ np2srv_tcp_keepalives(const char *client_name, const char *endpt_name, sr_sessio
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         if (!strcmp(node->schema->name, "idle-time")) {
             if (op == SR_OP_DELETED) {
                 idle_time = 1;
             } else {
-                idle_time = ((struct lyd_node_leaf_list *)node)->value.uint16;
+                idle_time = ((struct lyd_node_term *)node)->value.uint16;
             }
         } else if (!strcmp(node->schema->name, "max-probes")) {
             if (op == SR_OP_DELETED) {
                 max_probes = 10;
             } else {
-                max_probes = ((struct lyd_node_leaf_list *)node)->value.uint16;
+                max_probes = ((struct lyd_node_term *)node)->value.uint16;
             }
         } else if (!strcmp(node->schema->name, "probe-interval")) {
             if (op == SR_OP_DELETED) {
                 probe_interval = 5;
             } else {
-                probe_interval = ((struct lyd_node_leaf_list *)node)->value.uint16;
+                probe_interval = ((struct lyd_node_term *)node)->value.uint16;
             }
         }
     }
@@ -162,20 +164,19 @@ np2srv_tcp_keepalives(const char *client_name, const char *endpt_name, sr_sessio
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/ * /tcp-server-parameters */
 int
-np2srv_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_tcp_params_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
+    const char *endpt_name;
     char *xpath2;
-    bool prev_dflt;
     int rc, failed = 0;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -184,19 +185,19 @@ np2srv_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUSED(module_
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->child);
 
         if (!strcmp(node->schema->name, "local-address")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                if (nc_server_endpt_set_address(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str)) {
+                if (nc_server_endpt_set_address(endpt_name, lyd_get_value(node))) {
                     failed = 1;
                 }
             }
         } else if (!strcmp(node->schema->name, "local-port")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                if (nc_server_endpt_set_port(endpt_name, ((struct lyd_node_leaf_list *)node)->value.uint16)) {
+                if (nc_server_endpt_set_port(endpt_name, ((struct lyd_node_term *)node)->value.uint16)) {
                     failed = 1;
                 }
             }
@@ -216,7 +217,7 @@ np2srv_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUSED(module_
             /* set specific parameters */
             if (asprintf(&xpath2, "%s/keepalives/*", xpath) == -1) {
                 EMEM;
-                return SR_ERR_NOMEM;
+                return SR_ERR_NO_MEMORY;
             }
             if (np2srv_tcp_keepalives(NULL, endpt_name, session, xpath2)) {
                 failed = 1;
@@ -239,9 +240,8 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list;
-    bool prev_dflt;
     int rc;
+    time_t t;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
     if (rc != SR_ERR_OK) {
@@ -249,7 +249,7 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         if (!strcmp(node->schema->name, "period")) {
             if (op == SR_OP_DELETED) {
                 if (nc_server_ch_is_client(client_name)) {
@@ -257,7 +257,7 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
                     rc = nc_server_ch_client_periodic_set_period(client_name, 60);
                 }
             } else {
-                rc = nc_server_ch_client_periodic_set_period(client_name, ((struct lyd_node_leaf_list *)node)->value.uint16);
+                rc = nc_server_ch_client_periodic_set_period(client_name, ((struct lyd_node_term *)node)->value.uint16);
             }
         } else if (!strcmp(node->schema->name, "anchor-time")) {
             if (op == SR_OP_DELETED) {
@@ -266,8 +266,8 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
                     rc = nc_server_ch_client_periodic_set_anchor_time(client_name, 0);
                 }
             } else {
-                rc = nc_server_ch_client_periodic_set_anchor_time(client_name,
-                        nc_datetime2time(((struct lyd_node_leaf_list *)node)->value.string));
+                ly_time_str2time(lyd_get_value(node), &t, NULL);
+                rc = nc_server_ch_client_periodic_set_anchor_time(client_name, t);
             }
         } else if (!strcmp(node->schema->name, "idle-timeout")) {
             if (op == SR_OP_DELETED) {
@@ -277,7 +277,7 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
                 }
             } else {
                 rc = nc_server_ch_client_periodic_set_idle_timeout(client_name,
-                        ((struct lyd_node_leaf_list *)node)->value.uint16);
+                        ((struct lyd_node_term *)node)->value.uint16);
             }
         }
         if (rc) {
@@ -296,14 +296,13 @@ np2srv_ch_periodic_connection_params(const char *client_name, sr_session_ctx_t *
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client */
 int
-np2srv_ch_client_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
+np2srv_ch_client_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name), const char *xpath,
         sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *client_name;
-    bool prev_dflt;
+    const char *client_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -312,9 +311,9 @@ np2srv_ch_client_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get name */
-        client_name = ((struct lyd_node_leaf_list *)node->child)->value_str;
+        client_name = lyd_get_value(lyd_child(node));
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
@@ -341,20 +340,19 @@ np2srv_ch_client_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/ * /tcp-client-parameters */
 int
-np2srv_ch_client_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_tcp_params_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
+    const char *endpt_name, *client_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -363,21 +361,21 @@ np2srv_ch_client_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUS
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
 
         if (!strcmp(node->schema->name, "remote-address")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                if (nc_server_ch_client_endpt_set_address(client_name, endpt_name, ((struct lyd_node_leaf_list *)node)->value_str)) {
+                if (nc_server_ch_client_endpt_set_address(client_name, endpt_name, lyd_get_value(node))) {
                     sr_free_change_iter(iter);
                     return SR_ERR_INTERNAL;
                 }
             }
         } else if (!strcmp(node->schema->name, "remote-port")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                if (nc_server_ch_client_endpt_set_port(client_name, endpt_name, ((struct lyd_node_leaf_list *)node)->value.uint16)) {
+                if (nc_server_ch_client_endpt_set_port(client_name, endpt_name, ((struct lyd_node_term *)node)->value.uint16)) {
                     sr_free_change_iter(iter);
                     return SR_ERR_INTERNAL;
                 }
@@ -398,7 +396,7 @@ np2srv_ch_client_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUS
             /* set specific parameters */
             if (asprintf(&xpath2, "%s/keepalives/*", xpath) == -1) {
                 EMEM;
-                return SR_ERR_NOMEM;
+                return SR_ERR_NO_MEMORY;
             }
             rc = np2srv_tcp_keepalives(client_name, endpt_name, session, xpath2);
             free(xpath2);
@@ -419,20 +417,19 @@ np2srv_ch_client_endpt_tcp_params_cb(sr_session_ctx_t *session, const char *UNUS
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/connection-type */
 int
-np2srv_ch_connection_type_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_connection_type_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *client_name;
+    const char *client_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -441,9 +438,9 @@ np2srv_ch_connection_type_cb(sr_session_ctx_t *session, const char *UNUSED(modul
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find names */
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->child)->value_str;
+        client_name = lyd_get_value(node->parent->parent->child);
 
         /* connection type */
         if (op == SR_OP_CREATED) {
@@ -464,7 +461,7 @@ np2srv_ch_connection_type_cb(sr_session_ctx_t *session, const char *UNUSED(modul
         if (!strcmp(node->schema->name, "periodic")) {
             if (asprintf(&xpath2, "%s/periodic/*", xpath) == -1) {
                 EMEM;
-                return SR_ERR_NOMEM;
+                return SR_ERR_NO_MEMORY;
             }
             rc = np2srv_ch_periodic_connection_params(client_name, session, xpath2);
             free(xpath2);
@@ -484,20 +481,19 @@ np2srv_ch_connection_type_cb(sr_session_ctx_t *session, const char *UNUSED(modul
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/reconnect-strategy */
 int
-np2srv_ch_reconnect_strategy_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_reconnect_strategy_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *client_name, *str;
+    const char *client_name, *str;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -506,9 +502,9 @@ np2srv_ch_reconnect_strategy_cb(sr_session_ctx_t *session, const char *UNUSED(mo
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->child)->value_str;
+        client_name = lyd_get_value(node->parent->parent->child);
 
         if (!strcmp(node->schema->name, "start-with")) {
             if (op == SR_OP_DELETED) {
@@ -517,7 +513,7 @@ np2srv_ch_reconnect_strategy_cb(sr_session_ctx_t *session, const char *UNUSED(mo
                     rc = nc_server_ch_client_set_start_with(client_name, NC_CH_FIRST_LISTED);
                 }
             } else {
-                str = ((struct lyd_node_leaf_list *)node)->value_str;
+                str = lyd_get_value(node);
                 if (!strcmp(str, "first-listed")) {
                     rc = nc_server_ch_client_set_start_with(client_name, NC_CH_FIRST_LISTED);
                 } else if (!strcmp(str, "last-connected")) {
@@ -533,7 +529,7 @@ np2srv_ch_reconnect_strategy_cb(sr_session_ctx_t *session, const char *UNUSED(mo
                     rc = nc_server_ch_client_set_max_attempts(client_name, 3);
                 }
             } else {
-                rc = nc_server_ch_client_set_max_attempts(client_name, ((struct lyd_node_leaf_list *)node)->value.uint8);
+                rc = nc_server_ch_client_set_max_attempts(client_name, ((struct lyd_node_term *)node)->value.uint8);
             }
         }
 
