@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-netconf-server SSH callbacks
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -11,26 +13,30 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE /* asprintf() */
 #define _DEFAULT_SOURCE /* getpwent() */
 #define _POSIX_C_SOURCE 200809L /* getline() */
 
-#include <stdio.h>
-#include <stdint.h>
-#include <string.h>
+#include "netconf_server_ssh.h"
+
 #include <assert.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <pwd.h>
 #include <ctype.h>
+#include <errno.h>
+#include <pwd.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
 
 #include <libssh/libssh.h>
-#include <nc_server.h>
 #include <libyang/libyang.h>
+#include <nc_server.h>
 #include <sysrepo.h>
 
-#include "config.h"
 #include "common.h"
+#include "compat.h"
+#include "config.h"
 #include "log.h"
 #include "netconf_server.h"
 
@@ -71,7 +77,7 @@ np2srv_hostkey_cb(const char *name, void *UNUSED(user_data), char **UNUSED(privk
     rc = 0;
 
 cleanup:
-    lyd_free_withsiblings(data);
+    lyd_free_siblings(data);
     sr_session_stop(sr_sess);
     return rc;
 }
@@ -121,7 +127,7 @@ np2srv_pubkey_auth_cb(const struct nc_session *session, ssh_key key, void *UNUSE
 
         /* separate key type */
         ptr = line;
-        for (ptr2 = ptr; !isspace(ptr2[0]); ++ptr2);
+        for (ptr2 = ptr; !isspace(ptr2[0]); ++ptr2) {}
         if (ptr2[0] == '\0') {
             WRN("Invalid authorized key format of \"%s\" (line %d).", username, line_num);
             continue;
@@ -137,7 +143,7 @@ np2srv_pubkey_auth_cb(const struct nc_session *session, ssh_key key, void *UNUSE
 
         /* separate key data */
         ptr = ptr2 + 1;
-        for (ptr2 = ptr; !isspace(ptr2[0]); ++ptr2);
+        for (ptr2 = ptr; !isspace(ptr2[0]); ++ptr2) {}
         ptr2[0] = '\0';
 
         r = ssh_pki_import_pubkey_base64(ptr, ktype, &pub_key);
@@ -176,14 +182,13 @@ cleanup:
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/ssh */
 int
-np2srv_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_ssh_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
-    bool prev_dflt;
+    const char *endpt_name;
     int rc, failed = 0;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -192,9 +197,9 @@ np2srv_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
@@ -221,14 +226,13 @@ np2srv_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
 /* /ietf-netconf-server:netconf-server/listen/endpoint/ssh/ssh-server-parameters/server-identity/host-key/public-key/
  * keystore-reference */
 int
-np2srv_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
-    bool prev_dflt;
+    const char *prev_val, *endpt_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -237,19 +241,19 @@ np2srv_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(module
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
-            rc = nc_server_ssh_endpt_add_hostkey(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str, -1);
+            rc = nc_server_ssh_endpt_add_hostkey(endpt_name, lyd_get_value(node), -1);
         } else if (op == SR_OP_DELETED) {
             if (nc_server_is_endpt(endpt_name)) {
-                rc = nc_server_ssh_endpt_del_hostkey(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str, -1);
+                rc = nc_server_ssh_endpt_del_hostkey(endpt_name, lyd_get_value(node), -1);
             }
         } else if (op == SR_OP_MOVED) {
-            rc = nc_server_ssh_endpt_mov_hostkey(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str, prev_val);
+            rc = nc_server_ssh_endpt_mov_hostkey(endpt_name, lyd_get_value(node), prev_val);
         }
         if (rc) {
             sr_free_change_iter(iter);
@@ -268,7 +272,6 @@ np2srv_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(module
 static int
 np2srv_ssh_update_auth_method(const struct lyd_node *node, sr_change_oper_t op, int cur_auth)
 {
-    struct lyd_node_leaf_list *leaf;
     int auth;
 
     auth = cur_auth;
@@ -288,15 +291,14 @@ np2srv_ssh_update_auth_method(const struct lyd_node *node, sr_change_oper_t op,
     } else if (!strcmp(node->schema->name, "hostbased") || !strcmp(node->schema->name, "none")) {
         WRN("SSH authentication \"%s\" not supported.", node->schema->name);
     } else if (!strcmp(node->schema->name, "other")) {
-        leaf = (struct lyd_node_leaf_list *)node;
-        if (!strcmp(leaf->value_str, "interactive")) {
+        if (!strcmp(lyd_get_value(node), "interactive")) {
             if (op == SR_OP_CREATED) {
                 auth |= NC_SSH_AUTH_INTERACTIVE;
             } else if (op == SR_OP_DELETED) {
                 auth &= ~NC_SSH_AUTH_INTERACTIVE;
             }
         } else {
-            WRN("SSH authentication \"%s\" not supported.", leaf->value_str);
+            WRN("SSH authentication \"%s\" not supported.", lyd_get_value(node));
         }
     }
 
@@ -306,20 +308,19 @@ np2srv_ssh_update_auth_method(const struct lyd_node *node, sr_change_oper_t op,
 /* /ietf-netconf-server:netconf-server/listen/endpoint/ssh/ssh-server-parameters/client-authentication/
  * supported-authentication-methods */
 int
-np2srv_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
+    const char *endpt_name;
     char *xpath2;
-    bool prev_dflt;
     int rc, auth;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -328,9 +329,9 @@ np2srv_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, const char *UNUSED(m
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
 
         if ((op == SR_OP_DELETED) && !nc_server_is_endpt(endpt_name)) {
             /* endpt deleted */
@@ -364,14 +365,9 @@ np2srv_user_add_auth_key(const char *alg, size_t alg_len, const char *key, size_
     char name[7], *str;
     struct lyd_node *authkey;
 
-    authkey = lyd_new(user, NULL, "authorized-key");
-    if (!authkey) {
-        return -1;
-    }
-
-    /* name */
+    /* list with name */
     sprintf(name, "key%d", (*key_idx)++);
-    if (!lyd_new_leaf(authkey, NULL, "name", name)) {
+    if (lyd_new_list(user, NULL, "authorized-key", 0, &authkey, name)) {
         return -1;
     }
 
@@ -381,7 +377,7 @@ np2srv_user_add_auth_key(const char *alg, size_t alg_len, const char *key, size_
         EMEM;
         return -1;
     }
-    lyd_new_leaf(authkey, NULL, "algorithm", str);
+    lyd_new_term(authkey, NULL, "algorithm", str, 0, NULL);
     free(str);
 
     /* key-data */
@@ -390,7 +386,7 @@ np2srv_user_add_auth_key(const char *alg, size_t alg_len, const char *key, size_
         EMEM;
         return -1;
     }
-    lyd_new_leaf(authkey, NULL, "key-data", str);
+    lyd_new_term(authkey, NULL, "key-data", str, 0, NULL);
     free(str);
 
     return 0;
@@ -400,9 +396,9 @@ np2srv_user_add_auth_key(const char *alg, size_t alg_len, const char *key, size_
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/ssh/ssh-server-parameters/
  * client-authentication/users */
 int
-np2srv_endpt_ssh_auth_users_oper_cb(sr_session_ctx_t *UNUSED(session), const char *UNUSED(module_name),
-        const char *UNUSED(path), const char *UNUSED(request_xpath), uint32_t UNUSED(request_id), struct lyd_node **parent,
-        void *UNUSED(private_data))
+np2srv_endpt_ssh_auth_users_oper_cb(sr_session_ctx_t *UNUSED(session), uint32_t UNUSED(sub_id),
+        const char *UNUSED(module_name), const char *UNUSED(path), const char *UNUSED(request_xpath),
+        uint32_t UNUSED(request_id), struct lyd_node **parent, void *UNUSED(private_data))
 {
     struct passwd *pwd;
     struct lyd_node *users, *user;
@@ -412,27 +408,25 @@ np2srv_endpt_ssh_auth_users_oper_cb(sr_session_ctx_t *UNUSED(session), const cha
     int rc = SR_ERR_INTERNAL;
     uint8_t key_idx;
 
-    users = lyd_new(*parent, NULL, "users");
-    if (!users) {
+    if (lyd_new_inner(*parent, NULL, "users", 0, &users)) {
         return SR_ERR_INTERNAL;
     }
 
     while ((pwd = getpwent())) {
         /* create user with name */
-        user = lyd_new(users, NULL, "user");
-        if (!user) {
+        if (lyd_new_list(users, NULL, "user", 0, &user, pwd->pw_name)) {
             return SR_ERR_INTERNAL;
         }
-        lyd_new_leaf(user, NULL, "name", pwd->pw_name);
 
         /* check any authorized keys */
-        if (asprintf(&path, NP2SRV_SSH_AUTHORIZED_KEYS_PATTERN, NP2SRV_SSH_AUTHORIZED_KEYS_ARG_IS_USERNAME ? pwd->pw_name : pwd->pw_dir) == -1) {
+        if (asprintf(&path, NP2SRV_SSH_AUTHORIZED_KEYS_PATTERN, NP2SRV_SSH_AUTHORIZED_KEYS_ARG_IS_USERNAME ?
+                pwd->pw_name : pwd->pw_dir) == -1) {
             EMEM;
             goto cleanup;
         }
         f = fopen(path, "r");
         if (!f) {
-            if (errno != ENOENT && errno != ENOTDIR && errno != EACCES) {
+            if ((errno != ENOENT) && (errno != ENOTDIR) && (errno != EACCES)) {
                 ERR("Opening \"%s\" authorized key file failed (%s).", path, strerror(errno));
                 free(path);
                 goto cleanup;
@@ -509,14 +503,13 @@ cleanup:
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/ssh */
 int
-np2srv_ch_client_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_ssh_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
-    bool prev_dflt;
+    const char *endpt_name, *client_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -525,10 +518,10 @@ np2srv_ch_client_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(modu
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
@@ -555,14 +548,13 @@ np2srv_ch_client_endpt_ssh_cb(sr_session_ctx_t *session, const char *UNUSED(modu
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/ssh/ssh-server-parameters/
  * server-identity/host-key/public-key/keystore-reference */
 int
-np2srv_ch_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
-    bool prev_dflt;
+    const char *prev_val, *endpt_name, *client_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -571,23 +563,20 @@ np2srv_ch_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(mod
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->parent->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
-            rc = nc_server_ssh_ch_client_endpt_add_hostkey(client_name, endpt_name,
-                    ((struct lyd_node_leaf_list *)node)->value_str, -1);
+            rc = nc_server_ssh_ch_client_endpt_add_hostkey(client_name, endpt_name, lyd_get_value(node), -1);
         } else if (op == SR_OP_DELETED) {
             if (nc_server_ch_client_is_endpt(client_name, endpt_name)) {
-                rc = nc_server_ssh_ch_client_endpt_del_hostkey(client_name, endpt_name,
-                        ((struct lyd_node_leaf_list *)node)->value_str, -1);
+                rc = nc_server_ssh_ch_client_endpt_del_hostkey(client_name, endpt_name, lyd_get_value(node), -1);
             }
         } else if (op == SR_OP_MOVED) {
-            rc = nc_server_ssh_ch_client_endpt_mov_hostkey(client_name, endpt_name,
-                    ((struct lyd_node_leaf_list *)node)->value_str, prev_val);
+            rc = nc_server_ssh_ch_client_endpt_mov_hostkey(client_name, endpt_name, lyd_get_value(node), prev_val);
         }
         if (rc) {
             sr_free_change_iter(iter);
@@ -606,20 +595,19 @@ np2srv_ch_endpt_ssh_hostkey_cb(sr_session_ctx_t *session, const char *UNUSED(mod
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/ssh/ssh-server-parameters/
  * client-authentication/supported-authentication-methods */
 int
-np2srv_ch_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
+    const char *endpt_name, *client_name;
     char *xpath2;
-    bool prev_dflt;
     int rc, auth;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -628,10 +616,10 @@ np2srv_ch_endpt_ssh_auth_methods_cb(sr_session_ctx_t *session, const char *UNUSE
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->parent->child);
 
         if ((op == SR_OP_DELETED) && !nc_server_ch_client_is_endpt(client_name, endpt_name)) {
             continue;
