@@ -3,7 +3,9 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief ietf-netconf-server SSH callbacks
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * @copyright
+ * Copyright (c) 2019 - 2021 Deutsche Telekom AG.
+ * Copyright (c) 2017 - 2021 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -11,18 +13,22 @@
  *
  *     https://opensource.org/licenses/BSD-3-Clause
  */
+
 #define _GNU_SOURCE /* asprintf() */
 
+#include "netconf_server_tls.h"
+
+#include <assert.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
-#include <nc_server.h>
 #include <libyang/libyang.h>
+#include <nc_server.h>
 #include <sysrepo.h>
 
 #include "common.h"
+#include "compat.h"
 #include "log.h"
 #include "netconf_server.h"
 
@@ -61,11 +67,11 @@ np2srv_cert_cb(const char *name, void *UNUSED(user_data), char **UNUSED(cert_pat
 
     /* get cert data from sysrepo */
     if (asprintf(&xpath, "/ietf-keystore:keystore/asymmetric-keys/asymmetric-key/certificates/"
-                "certificate[name='%s']/cert", name) == -1) {
+            "certificate[name='%s']/cert", name) == -1) {
         EMEM;
         goto cleanup;
     }
-    lyd_free_withsiblings(data);
+    lyd_free_siblings(data);
     r = sr_get_subtree(sr_sess, xpath, 0, &data);
     free(xpath);
     if (r != SR_ERR_OK) {
@@ -76,7 +82,7 @@ np2srv_cert_cb(const char *name, void *UNUSED(user_data), char **UNUSED(cert_pat
     }
 
     /* set cert data */
-    *cert_data = strdup(((struct lyd_node_leaf_list *)data)->value_str);
+    *cert_data = strdup(lyd_get_value(data));
     if (!*cert_data) {
         EMEM;
         goto cleanup;
@@ -86,7 +92,7 @@ np2srv_cert_cb(const char *name, void *UNUSED(user_data), char **UNUSED(cert_pat
     rc = 0;
 
 cleanup:
-    lyd_free_withsiblings(data);
+    lyd_free_siblings(data);
     sr_session_stop(sr_sess);
     return rc;
 }
@@ -122,25 +128,24 @@ np2srv_cert_list_cb(const char *name, void *UNUSED(user_data), char ***UNUSED(ce
     }
 
     /* find all certificates */
-    set = lyd_find_path(data, "certificate/cert");
-    if (!set) {
+    if (lyd_find_xpath(data, "certificate/cert", &set)) {
         /* libyang error printed */
         goto cleanup;
-    } else if (!set->number) {
+    } else if (!set->count) {
         WRN("Certificate list \"%s\" does not define any actual certificates.");
         rc = 0;
         goto cleanup;
     }
 
-    *cert_data = malloc(set->number * sizeof **cert_data);
+    *cert_data = malloc(set->count * sizeof **cert_data);
     if (!*cert_data) {
         EMEM;
         goto cleanup;
     }
 
     /* collect all cert data */
-    for (i = 0; i < set->number; ++i) {
-        (*cert_data)[i] = strdup(((struct lyd_node_leaf_list *)set->set.d[i])->value_str);
+    for (i = 0; i < set->count; ++i) {
+        (*cert_data)[i] = strdup(lyd_get_value(set->dnodes[i]));
         if (!(*cert_data)[i]) {
             EMEM;
             for (j = 0; j < i - 1; ++j) {
@@ -150,28 +155,27 @@ np2srv_cert_list_cb(const char *name, void *UNUSED(user_data), char ***UNUSED(ce
             goto cleanup;
         }
     }
-    *cert_data_count = set->number;
+    *cert_data_count = set->count;
 
     /* success */
     rc = 0;
 
 cleanup:
-    lyd_free_withsiblings(data);
-    ly_set_free(set);
+    lyd_free_siblings(data);
+    ly_set_free(set, NULL);
     sr_session_stop(sr_sess);
     return rc;
 }
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/tls */
 int
-np2srv_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_tls_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
-    bool prev_dflt;
+    const char *endpt_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -180,9 +184,9 @@ np2srv_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
@@ -206,20 +210,19 @@ np2srv_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(module_name),
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/tls/tls-server-parameters/server-identity/keystore-reference */
 int
-np2srv_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_tls_servercert_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
+    const char *endpt_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -228,14 +231,14 @@ np2srv_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *UNUSED(mod
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
 
         /* we do not care about the "asymmetric-key", the certificate is enough */
         if (!strcmp(node->schema->name, "certificate")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                rc = nc_server_tls_endpt_set_server_cert(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_endpt_set_server_cert(endpt_name, lyd_get_value(node));
             } else if (op == SR_OP_DELETED) {
                 if (nc_server_is_endpt(endpt_name)) {
                     rc = nc_server_tls_endpt_set_server_cert(endpt_name, NULL);
@@ -258,20 +261,19 @@ np2srv_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *UNUSED(mod
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/tls/tls-server-parameters/client-authentication */
 int
-np2srv_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_tls_client_auth_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name;
+    const char *prev_val, *endpt_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -280,9 +282,9 @@ np2srv_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char *UNUSED(mo
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->child);
 
         if (!strcmp(node->schema->name, "optional")) {
             /* it is always required */
@@ -291,14 +293,14 @@ np2srv_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char *UNUSED(mo
             return SR_ERR_UNSUPPORTED;
         } else if (!strcmp(node->schema->name, "ca-certs") || !strcmp(node->schema->name, "client-certs")) {
             if (op == SR_OP_CREATED) {
-                rc = nc_server_tls_endpt_add_trusted_cert_list(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_endpt_add_trusted_cert_list(endpt_name, lyd_get_value(node));
             } else if (op == SR_OP_DELETED) {
                 if (nc_server_is_endpt(endpt_name)) {
-                    rc = nc_server_tls_endpt_del_trusted_cert_list(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str);
+                    rc = nc_server_tls_endpt_del_trusted_cert_list(endpt_name, lyd_get_value(node));
                 }
             } else if (op == SR_OP_MODIFIED) {
                 nc_server_tls_endpt_del_trusted_cert_list(endpt_name, prev_val);
-                rc = nc_server_tls_endpt_add_trusted_cert_list(endpt_name, ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_endpt_add_trusted_cert_list(endpt_name, lyd_get_value(node));
             }
             if (rc) {
                 sr_free_change_iter(iter);
@@ -344,22 +346,21 @@ np2srv_tls_ctn_str2map_type(const char *map_type)
 
 /* /ietf-netconf-server:netconf-server/listen/endpoint/tls/tls-server-parameters/client-authentication/cert-maps */
 int
-np2srv_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node, *child;
-    const char *prev_val, *prev_list, *endpt_name, *fingerprint, *name;
+    const char *endpt_name, *fingerprint, *name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
     uint32_t id;
     NC_TLS_CTN_MAPTYPE map_type;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -368,24 +369,24 @@ np2srv_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, const char *UNUSED(mod
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* find name */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
 
         /* collect all attributes */
         id = 0;
         fingerprint = NULL;
         map_type = 0;
         name = NULL;
-        LY_TREE_FOR(node->child, child) {
+        LY_LIST_FOR(lyd_child(node), child) {
             if (!strcmp(child->schema->name, "id")) {
-                id = ((struct lyd_node_leaf_list *)child)->value.uint32;
+                id = ((struct lyd_node_term *)child)->value.uint32;
             } else if (!strcmp(child->schema->name, "fingerprint")) {
-                fingerprint = ((struct lyd_node_leaf_list *)child)->value_str;
+                fingerprint = lyd_get_value(child);
             } else if (!strcmp(child->schema->name, "map-type")) {
-                map_type = np2srv_tls_ctn_str2map_type(((struct lyd_node_leaf_list *)child)->value_str);
+                map_type = np2srv_tls_ctn_str2map_type(lyd_get_value(child));
             } else if (!strcmp(child->schema->name, "name")) {
-                name = ((struct lyd_node_leaf_list *)child)->value_str;
+                name = lyd_get_value(child);
             }
         }
         /* it was validated */
@@ -417,14 +418,13 @@ np2srv_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, const char *UNUSED(mod
 
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/tls */
 int
-np2srv_ch_client_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_tls_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id), const char *UNUSED(module_name),
+        const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
-    bool prev_dflt;
+    const char *endpt_name, *client_name;
     int rc;
 
     rc = sr_get_changes_iter(session, xpath, &iter);
@@ -433,10 +433,10 @@ np2srv_ch_client_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(modu
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->child);
 
         /* ignore other operations */
         if (op == SR_OP_CREATED) {
@@ -463,20 +463,20 @@ np2srv_ch_client_endpt_tls_cb(sr_session_ctx_t *session, const char *UNUSED(modu
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/tls/tls-server-parameters/"
  * server-identity/keystore-reference */
 int
-np2srv_ch_client_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_tls_servercert_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id),
+        const char *UNUSED(module_name), const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id),
+        void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
+    const char *endpt_name, *client_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -485,16 +485,15 @@ np2srv_ch_client_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->parent->child);
 
         /* we do not care about the "asymmetric-key", the certificate is enough */
         if (!strcmp(node->schema->name, "certificate")) {
             if ((op == SR_OP_CREATED) || (op == SR_OP_MODIFIED)) {
-                rc = nc_server_tls_ch_client_endpt_set_server_cert(client_name, endpt_name,
-                        ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_ch_client_endpt_set_server_cert(client_name, endpt_name, lyd_get_value(node));
             } else if (op == SR_OP_DELETED) {
                 if (nc_server_ch_client_is_endpt(client_name, endpt_name)) {
                     rc = nc_server_tls_ch_client_endpt_set_server_cert(client_name, endpt_name, NULL);
@@ -518,20 +517,20 @@ np2srv_ch_client_endpt_tls_servercert_cb(sr_session_ctx_t *session, const char *
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/tls/tls-server-parameters/"
  * client-authentication */
 int
-np2srv_ch_client_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_tls_client_auth_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id),
+        const char *UNUSED(module_name), const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id),
+        void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node;
-    const char *prev_val, *prev_list, *endpt_name, *client_name;
+    const char *prev_val, *endpt_name, *client_name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -540,10 +539,10 @@ np2srv_ch_client_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, NULL, NULL)) == SR_ERR_OK) {
         /* get names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->child);
 
         if (!strcmp(node->schema->name, "optional")) {
             /* it is always required */
@@ -552,17 +551,14 @@ np2srv_ch_client_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char
             return SR_ERR_UNSUPPORTED;
         } else if (!strcmp(node->schema->name, "ca-certs") || !strcmp(node->schema->name, "client-certs")) {
             if (op == SR_OP_CREATED) {
-                rc = nc_server_tls_ch_client_endpt_add_trusted_cert_list(client_name, endpt_name,
-                        ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_ch_client_endpt_add_trusted_cert_list(client_name, endpt_name, lyd_get_value(node));
             } else if (op == SR_OP_DELETED) {
                 if (nc_server_ch_client_is_endpt(client_name, endpt_name)) {
-                    rc = nc_server_tls_ch_client_endpt_del_trusted_cert_list(client_name, endpt_name,
-                            ((struct lyd_node_leaf_list *)node)->value_str);
+                    rc = nc_server_tls_ch_client_endpt_del_trusted_cert_list(client_name, endpt_name, lyd_get_value(node));
                 }
             } else if (op == SR_OP_MODIFIED) {
                 nc_server_tls_ch_client_endpt_del_trusted_cert_list(client_name, endpt_name, prev_val);
-                rc = nc_server_tls_ch_client_endpt_add_trusted_cert_list(client_name, endpt_name,
-                        ((struct lyd_node_leaf_list *)node)->value_str);
+                rc = nc_server_tls_ch_client_endpt_add_trusted_cert_list(client_name, endpt_name, lyd_get_value(node));
             }
             if (rc) {
                 sr_free_change_iter(iter);
@@ -582,22 +578,22 @@ np2srv_ch_client_endpt_tls_client_auth_cb(sr_session_ctx_t *session, const char
 /* /ietf-netconf-server:netconf-server/call-home/netconf-client/endpoints/endpoint/tls/tls-server-parameters/"
  * client-authentication/cert-maps */
 int
-np2srv_ch_client_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, const char *UNUSED(module_name), const char *xpath,
-        sr_event_t UNUSED(event), uint32_t UNUSED(request_id), void *UNUSED(private_data))
+np2srv_ch_client_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, uint32_t UNUSED(sub_id),
+        const char *UNUSED(module_name), const char *xpath, sr_event_t UNUSED(event), uint32_t UNUSED(request_id),
+        void *UNUSED(private_data))
 {
     sr_change_iter_t *iter;
     sr_change_oper_t op;
     const struct lyd_node *node, *child;
-    const char *prev_val, *prev_list, *endpt_name, *client_name, *fingerprint, *name;
+    const char *endpt_name, *client_name, *fingerprint, *name;
     char *xpath2;
-    bool prev_dflt;
     int rc;
     uint32_t id;
     NC_TLS_CTN_MAPTYPE map_type;
 
     if (asprintf(&xpath2, "%s/*", xpath) == -1) {
         EMEM;
-        return SR_ERR_NOMEM;
+        return SR_ERR_NO_MEMORY;
     }
     rc = sr_get_changes_iter(session, xpath2, &iter);
     free(xpath2);
@@ -606,25 +602,25 @@ np2srv_ch_client_endpt_tls_client_ctn_cb(sr_session_ctx_t *session, const char *
         return rc;
     }
 
-    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, &prev_val, &prev_list, &prev_dflt)) == SR_ERR_OK) {
+    while ((rc = sr_get_change_tree_next(session, iter, &op, &node, NULL, NULL, NULL)) == SR_ERR_OK) {
         /* get names */
-        endpt_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->child)->value_str;
-        client_name = ((struct lyd_node_leaf_list *)node->parent->parent->parent->parent->parent->parent->parent->child)->value_str;
+        endpt_name = lyd_get_value(node->parent->parent->parent->parent->parent->child);
+        client_name = lyd_get_value(node->parent->parent->parent->parent->parent->parent->parent->child);
 
         /* collect all attributes */
         id = 0;
         fingerprint = NULL;
         map_type = 0;
         name = NULL;
-        LY_TREE_FOR(node->child, child) {
+        LY_LIST_FOR(lyd_child(node), child) {
             if (!strcmp(child->schema->name, "id")) {
-                id = ((struct lyd_node_leaf_list *)child)->value.uint32;
+                id = ((struct lyd_node_term *)child)->value.uint32;
             } else if (!strcmp(child->schema->name, "fingerprint")) {
-                fingerprint = ((struct lyd_node_leaf_list *)child)->value_str;
+                fingerprint = lyd_get_value(child);
             } else if (!strcmp(child->schema->name, "map-type")) {
-                map_type = np2srv_tls_ctn_str2map_type(((struct lyd_node_leaf_list *)child)->value_str);
+                map_type = np2srv_tls_ctn_str2map_type(lyd_get_value(child));
             } else if (!strcmp(child->schema->name, "name")) {
-                name = ((struct lyd_node_leaf_list *)child)->value_str;
+                name = lyd_get_value(child);
             }
         }
         /* it was validated */
